//#include "../../../code/vc6/alpha1/src/blepo.h"
//#include "../../../code/vc6/alpha1/src/Image/EquivalenceTable.h"
#include <vector>

using namespace blepo;

struct LineParameter
{
	double a;
	double b;
	double c;
	double d;
};

struct SeedPoint
{
	int x;
	int y;
	double val;
	int width;
};

struct RootElement
{
	double theta;
	int length;
	int id;
	double width;
	double val;
	std::vector<int> points_no;
	std::vector<int> points_x;
	std::vector<int> points_y;
	std::vector<int> valid_points_x;
	std::vector<int> valid_points_y;
	std::vector<int> centeraxis_points_x;
	std::vector<int> centeraxis_points_y;
	std::vector<double> line_parameter;
	std::vector<SeedPoint> end_points;
	std::vector<LineParameter> total_line_para;
	blepo::ImgGray root_area;
	int hist_number;
	int als_number;
	int pb_number;
	double eccentricity;
	double ratio_hist;
	double ratio_edge;
	double ratio_als;
	double ratio_pb;
	int root_or_not;
};



void Otsu(ImgGray& img, unsigned char* threshold_val);
void Sobel(const ImgGray& img, ImgGray* out);

bool pop(int &x, int &y, int w) ;
bool push(int x, int y, int w) ;
void emptyStack(int x, int y, int w) ;
void floodFill4Stack(ImgGray& img, int x, int y, int newColor, int oldColor, int half_width, std::vector<LineParameter>& line_para, std::vector<int>* flood_points_x, std::vector<int>* flood_points_y);
void floodFill4Binary(ImgGray& img, int x, int y, int newColor, int oldColor, int half_width, std::vector<double>& line_para, std::vector<int>* flood_points_x, std::vector<int>* flood_points_y);

void FindSkelton(const ImgGray& root_half, std::vector<int>* SkeletonX, std::vector<int>* SkeletonY);
void FindLengthPath(ImgGray& RootArea, double AreaOrient, std::vector<SeedPoint>* centerline_points);
void FindDiameter(ImgGray& RootArea, std::vector<SeedPoint>& centeraxis_points, double* root_diameter, int* salient);

void GreyFloodFill(RootElement& root_centerline, ImgGray& grey_img, double* cover_percent,  std::vector<SeedPoint>* cover_points, std::vector<SeedPoint>* not_cover_points, std::vector<int>* filled_points_x, std::vector<int>* filled_points_y);
void RegionClean(std::vector<int>& filled_point_x, std::vector<int>& filled_point_y, int img_width, int img_height, ImgGray* temp_binary_2, std::vector<SeedPoint>* filled_clean_points);
void RegionPropsExtract(ImgGray& root_region, std::vector<SeedPoint>& filled_clean_points, RootElement* temp_region);
void BrokenRegionCombine(RootElement& root_old, ImgGray& img_1, ImgGray& img_2, std::vector<SeedPoint>& OldPoints, std::vector<SeedPoint>& NewPoints, ImgGray* img_3);
void RootsRegionGrowing(const std::vector<RootElement>& clean_root, ImgGray& GreyImg, std::vector<RootElement>* grown_root);

void LineFitting(const MatDbl& line_points, std::vector<double>* line_para);
void RootSelect(const std::vector<SeedPoint>& seed_pp, std::vector<RootElement>* points_pair);
void FindTwoEnd(RootElement& line_1, RootElement& line_2, std::vector<SeedPoint>& one_end, std::vector<SeedPoint>* two_end);
void LineDist(RootElement& line_1, RootElement& line_2, double* dist_lines);
void RootsCombine(std::vector<RootElement>& clean_roots, std::vector<RootElement>* combined_roots);
void BorderCheck(RootElement& tmp, int* valid);
void RootsClean(std::vector<RootElement>& points_pair, std::vector<SeedPoint>& clean_seed, std::vector<RootElement>* root_points);

void WriteMoreData(CString& file_name, const ImgGray& img_r, const ImgGray& img_g, const ImgGray& img_b, int img_size_x, int img_size_y, std::vector<int>& seed_rr_x, std::vector<int>& seed_rr_y);
void WriteData(CString& file_name, const ImgGray& img_r, const ImgGray& img_g, const ImgGray& img_b, int img_size_x, int img_size_y, std::vector<int>& seed_rr_x, std::vector<int>& seed_rr_y);

void FindMaxima(const ImgFloat& img, int search_type, int skip_axis_val, double min_peak, double min_width, int scale, std::vector<SeedPoint>* maxima);
void SelectSeedPointsHorizontal(const ImgGray& sample_img, const ImgFloat& gauss_w, double skip_dist, double mountain_high, double mountain_wide, int scale, std::vector<SeedPoint>* hor_seed);
void SelectSeedPointsVertical(const ImgGray& sample_img, const ImgFloat& gauss_h, double skip_dist, double mountain_high, double mountain_wide, int scale, std::vector<SeedPoint>* ver_seed);
void SeedPointsCombineHorizontal(std::vector<SeedPoint>& horz_seed, int scale, std::vector<SeedPoint>* horz_total);
void SeedPointsCombineVertical(std::vector<SeedPoint>& vert_seed, int scale, std::vector<SeedPoint>* vert_total);
void SeedPointSelection(const ImgGray& root_img, std::vector<SeedPoint>* hh_seed, std::vector<SeedPoint>* vv_seed);

void RegionProps(ImgGray& tmp, RootElement* temp_refine);
void HistogramVal(ImgGray& img_half, RootElement& root1, int* hist_no);
void EdgeVal(ImgGray& img_half, RootElement& root1, int* edge_no);
void FindFeatures(ImgGray& RootArea, std::vector<SeedPoint>& centeraxis, double* root_diameter, int* als_no, int* pb_no);

void RootsRegionRefine(ImgGray& img_half, std::vector<RootElement>&grown_roots, std::vector<RootElement>* refine_roots);
void RootClassify(ImgGray& img_half, RootElement& region, int* root_true);
void DetectRoots(const ImgBgr& img_load, int ff, std::vector<RootElement>* RefineRoots);

            



